@@ -7,6 +7,7 @@
 # @WeChat       : 313303303
 # @Software     : PyCharm
 # @Description  :
+from meutils.pipe import *
 
 
 def check_classification(y, threshold=128):
@@ -19,7 +20,7 @@ def check_classification(y, threshold=128):
     return len(set(y)) < threshold
 
 
-def check_pandas_input(self, arg):
+def check_pandas_input(arg):
     try:
         return arg.values
     except AttributeError:
@@ -28,6 +29,9 @@ def check_pandas_input(self, arg):
         )
 
 
-def infer_cat_feats(df, threshold=64):
-    threshold = min(len(df) // 100 + 8, threshold)
-    return df.nunique()[lambda x: x <= threshold].index.tolist()
+def check_category(s: pd.Series, threshold=64):
+    return s.nunique(dropna=False) <= threshold
+
+
+def infer_cat_feats(df: pd.DataFrame, threshold=64):
+    return df.nunique(dropna=False)[lambda x: x <= threshold].index.tolist()
