@@ -6,10 +6,11 @@
 # @Author       : yuanjie
 # @Email        : yuanjie@xiaomi.com
 # @Software     : PyCharm
-# @Description  :
+# @Description  : todo: 增加 lag
 
 
 from meutils.pipe import *
+from pandas.tseries.holiday import USFederalHolidayCalendar
 
 
 class DateTimeFeats(object):
@@ -50,10 +51,17 @@ class DateTimeFeats(object):
         self.feats = feats
         if feats is None:
             self.feats = (
+                'timestamp',  # 获取时间戳
                 "year", "quarter", "month", "day", "hour", "minute", "second",
                 "week", "weekday",  # week==weekofyear== .dt.isocalendar().week
                 "day_of_year",
-                "days_in_month",  # 月的天数
+
+                # 自定义
+                "monthweek",  # 获取指定的某天是某个月的第几周
+                'day_part',  # 拆分为早中晚
+
+                'holiday_name',  #
+                'solar_terms',  # 节气
 
                 # bool
                 'is_month_end',
@@ -62,17 +70,13 @@ class DateTimeFeats(object):
                 'is_quarter_start',
                 'is_year_end',
                 'is_year_start',
-                'is_leap_year',
                 'is_weekend',  #
                 'is_holiday',  #
+                'is_holiday4us',
                 'is_in_lieu',  #
                 'is_special_day',  #
-
-                # 自定义
-                "monthweek",  # 获取指定的某天是某个月的第几周
-                'solar_terms',  # 节气
-                'holiday_name',  #
-                'day_part',  # 拆分为早中晚
+                # 'is_leap_year',
+                # "days_in_month",  # 月的天数
 
             )
 
@@ -86,10 +90,10 @@ class DateTimeFeats(object):
             else:
                 _ = self.__getattribute__(feat)(s)
 
-            if _.dtypes == 'bool':
+            if _.dtype == 'bool':
                 _ = _.astype(int)
 
-            df[f"{s.name}_{feat}"] = _
+            df[f"{s.name}_{feat}"] = _  # lag
 
         return df
 
@@ -125,12 +129,20 @@ class DateTimeFeats(object):
 
         return s.map(is_holiday)
 
+    @staticmethod
+    def is_holiday4us(s: pd.Series):
+        us_holidays = USFederalHolidayCalendar().holidays(start=s.dt.date.min(), end=s.dt.date.max())
+        return s.isin(us_holidays)
+
     @staticmethod
     def is_in_lieu(s: pd.Series):  # 调休日
         from chinese_calendar import is_holiday, is_workday, is_in_lieu, get_holiday_detail, get_solar_terms
 
         return s.map(is_in_lieu)
 
+    def is_solar_terms(self, s: pd.Series):  # 节气
+        return self.solar_terms(s).map(lambda x: 0 if x is np.nan else 1)
+
     @staticmethod
     def solar_terms(s: pd.Series):  # 节气
         from chinese_calendar import is_holiday, is_workday, is_in_lieu, get_holiday_detail, get_solar_terms
@@ -162,29 +174,33 @@ class DateTimeFeats(object):
         @lru_cache(32)
         def f(hour):
             if hour in [4, 5]:
-                return "dawn"
+                return 0  # "dawn"
             elif hour in [6, 7]:
-                return "early morning"
+                return 1  # "early morning"
             elif hour in [8, 9, 10]:
-                return "late morning"
+                return 2  # "late morning"
             elif hour in [11, 12, 13]:
-                return "noon"
+                return 3  # "noon"
             elif hour in [14, 15, 16]:
-                return "afternoon"
+                return 4  # "afternoon"
             elif hour in [17, 18, 19]:
-                return "evening"
+                return 5  # "evening"
             elif hour in [20, 21, 22]:
-                return "night"
+                return 6  # "night"
             elif hour in [23, 24, 1, 2, 3]:
-                return "midnight"
+                return 7  # "midnight"
             else:
                 return np.nan
 
         return s.dt.hour.map(f)
 
+    @staticmethod
+    def timestamp(s: pd.Series):
+        return pd.DatetimeIndex(s).asi8 // 10 ** 9  # s.astype(np.int64) // 10**9
+
 
 if __name__ == '__main__':
     import pandas as pd
 
     ts = pd.date_range('2022-10-01', '2022-10-30', freq='h').to_series()
-    print(DateTimeFeats(['day_part']).transform(ts))
+    print(DateTimeFeats(['is_holiday4us']).transform(ts))
